#include <pthread.h>
#include <rtai.h>
#include <rtai_sched.h>
#include <rtai_msg.h>
#include <rtai_mbx.h>
#include <rtai_usi.h>
#include <rtai_sem.h>
#include <sched.h>
#include <sys/io.h>


#define PAR_BASE   (0x3BC)
#define PAR_STATUS (PAR_BASE + 1)
#define PAR_CTRL   (PAR_BASE + 2)

volatile int counter;

const int TimerIrq = 7;

void *thread1( void* arg )
{
	void* result = NULL;
	RT_TASK *task1 = NULL;

	/* Create new realtime task */
	task1 = rt_task_init_schmod( nam2num("MYTHD1"),	/* ID */
								0,					/* Priority: 0 is highest.
														0x3fffffff is lowest */
								1024,				/* Stack size */
								1024,				/* Max message size */
								SCHED_FIFO,			/* Task scheduling policy */
								1					/* Processor affinity */
								);
	if( NULL == task1 )
	{
		printf("Failed to create task1\n");
		return( result );
	}

	/* Make this task realtime */
	mlockall( MCL_CURRENT | MCL_FUTURE );
	rt_make_hard_real_time();

	rt_request_irq_task( TimerIrq, task1, RT_IRQ_TASK, 1 );
	rt_startup_irq( TimerIrq );
	rt_enable_irq( TimerIrq );

	/* do realtime job */
	while( counter < 10 )
	{
		rt_irq_wait( TimerIrq );
		++counter;
		outb( 0x00, PAR_BASE );
		/* enable interrupt */
		inb( PAR_STATUS );
		rt_ack_irq( TimerIrq );
	}

	rt_pend_linux_irq( TimerIrq );
	rt_release_irq_task( TimerIrq );
	
	/* Delete realtime task */
	rt_task_delete( task1 );

	/* Exit thread */
	return( result );
}

main()
{
	pthread_t th1;
	RT_TASK *mainTask = NULL;
	int i = 0;

	/* Enable non-root hrt */
	rt_allow_nonroot_hrt();
	ioperm( PAR_BASE, 3, 1 );

	/* Set parallel port to output */
	outb( (1<<4), PAR_CTRL );
	inb( PAR_STATUS );

	/* Create realtime task */
	mainTask = rt_task_init_schmod( nam2num("MAINTH"),	/* ID */
								10,					/* Priority: 0 is highest.
														0x3fffffff is lowest */
								1024,				/* Stack size */
								1024,				/* Max message size */
								SCHED_FIFO,			/* Task scheduling policy */
								1					/* Processor affinity */
								);
	if( NULL == mainTask )
	{
		printf("Failed to create main task.\n");
		return( 0 );
	}

	/* Create user thread */
	pthread_create( &th1, NULL, thread1, NULL );

	outb( 0x00, PAR_BASE );
	for( i = 0 ; i < 10 ; ++i )
	{
		sleep( 1 );
		outb( 0xFF, PAR_BASE );
		sleep( 1 );
		outb( 0x00, PAR_BASE );
		printf("counter = %d\n", counter );
	}

	pthread_join( th1, NULL );

	/* Delete MAINTH */
	rt_task_delete( mainTask );

}


